@@ -79,6 +79,14 @@ struct _GeditViewFramePrivate
 	 * cancelled.
 	 */
 	GtkSourceSearchSettings *old_search_settings;
+
+	/* The original search texts. In search_settings and
+	 * old_search_settings, the search text is unescaped. Since the escape
+	 * function is not reciprocal, we need to store the original search
+	 * texts.
+	 */
+	gchar *search_text;
+	gchar *old_search_text;
 };
 
 enum
@@ -114,6 +122,17 @@ gedit_view_frame_dispose (GObject *object)
 	G_OBJECT_CLASS (gedit_view_frame_parent_class)->dispose (object);
 }
 
+static void
+gedit_view_frame_finalize (GObject *object)
+{
+	GeditViewFrame *frame = GEDIT_VIEW_FRAME (object);
+
+	g_free (frame->priv->search_text);
+	g_free (frame->priv->old_search_text);
+
+	G_OBJECT_CLASS (gedit_view_frame_parent_class)->finalize (object);
+}
+
 static void
 gedit_view_frame_get_property (GObject    *object,
                                guint       prop_id,
@@ -242,10 +261,16 @@ set_search_state (GeditViewFrame *frame,
 	if (state == SEARCH_STATE_NOT_FOUND)
 	{
 		gtk_style_context_add_class (context, "not-found");
+
+		gtk_widget_set_sensitive (frame->priv->go_down_button, FALSE);
+		gtk_widget_set_sensitive (frame->priv->go_up_button, FALSE);
 	}
 	else
 	{
 		gtk_style_context_remove_class (context, "not-found");
+
+		gtk_widget_set_sensitive (frame->priv->go_down_button, TRUE);
+		gtk_widget_set_sensitive (frame->priv->go_up_button, TRUE);
 	}
 }
 
@@ -533,6 +558,14 @@ search_widget_key_press_event (GtkWidget      *widget,
 
 			gtk_source_search_context_set_settings (search_context,
 								frame->priv->search_settings);
+
+			g_free (frame->priv->search_text);
+			frame->priv->search_text = NULL;
+
+			if (frame->priv->old_search_text != NULL)
+			{
+				frame->priv->search_text = g_strdup (frame->priv->old_search_text);
+			}
 		}
 
 		hide_search_widget (frame, TRUE);
@@ -617,25 +650,18 @@ update_entry_tag (GeditViewFrame *frame)
 		return;
 	}
 
-	if (count == 0)
+	if (count == 0 || pos == 0)
 	{
 		gd_tagged_entry_remove_tag (frame->priv->search_entry,
 					    frame->priv->entry_tag);
 		return;
 	}
 
-	if (pos == 0)
-	{
-		label = g_strdup_printf ("%d", count);
-	}
-	else
-	{
-		/* Translators: the first %d is the position of the current search
-		 * occurrence, and the second %d is the total number of search
-		 * occurrences.
-		 */
-		label = g_strdup_printf (_("%d of %d"), pos, count);
-	}
+	/* Translators: the first %d is the position of the current search
+	 * occurrence, and the second %d is the total number of search
+	 * occurrences.
+	 */
+	label = g_strdup_printf (_("%d of %d"), pos, count);
 
 	gd_tagged_entry_tag_set_label (frame->priv->entry_tag, label);
 
@@ -668,16 +694,55 @@ install_update_entry_tag_idle (GeditViewFrame *frame)
 static void
 update_search_text (GeditViewFrame *frame)
 {
-	const gchar *entry_text;
-	gchar *unescaped_entry_text;
+	const gchar *entry_text = gtk_entry_get_text (GTK_ENTRY (frame->priv->search_entry));
 
-	entry_text = gtk_entry_get_text (GTK_ENTRY (frame->priv->search_entry));
-	unescaped_entry_text = gtk_source_utils_unescape_search_text (entry_text);
+	g_free (frame->priv->search_text);
+	frame->priv->search_text = g_strdup (entry_text);
 
-	gtk_source_search_settings_set_search_text (frame->priv->search_settings,
-						    unescaped_entry_text);
+	if (gtk_source_search_settings_get_regex_enabled (frame->priv->search_settings))
+	{
+		gtk_source_search_settings_set_search_text (frame->priv->search_settings,
+							    entry_text);
+	}
+	else
+	{
+		gchar *unescaped_entry_text = gtk_source_utils_unescape_search_text (entry_text);
 
-	g_free (unescaped_entry_text);
+		gtk_source_search_settings_set_search_text (frame->priv->search_settings,
+							    unescaped_entry_text);
+
+		g_free (unescaped_entry_text);
+	}
+}
+
+static void
+regex_toggled_cb (GtkCheckMenuItem *menu_item,
+		  GeditViewFrame   *frame)
+{
+	gtk_source_search_settings_set_regex_enabled (frame->priv->search_settings,
+						      gtk_check_menu_item_get_active (menu_item));
+
+	start_search (frame);
+}
+
+static void
+at_word_boundaries_toggled_cb (GtkCheckMenuItem *menu_item,
+			       GeditViewFrame   *frame)
+{
+	gtk_source_search_settings_set_at_word_boundaries (frame->priv->search_settings,
+							   gtk_check_menu_item_get_active (menu_item));
+
+	start_search (frame);
+}
+
+static void
+case_sensitive_toggled_cb (GtkCheckMenuItem *menu_item,
+			   GeditViewFrame   *frame)
+{
+	gtk_source_search_settings_set_case_sensitive (frame->priv->search_settings,
+						       gtk_check_menu_item_get_active (menu_item));
+
+	start_search (frame);
 }
 
 static void
@@ -685,6 +750,7 @@ add_popup_menu_items (GeditViewFrame *frame,
 		      GtkWidget      *menu)
 {
 	GtkWidget *menu_item;
+	gboolean val;
 
 	/* create "Wrap Around" menu item. */
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("_Wrap Around"));
@@ -702,9 +768,13 @@ add_popup_menu_items (GeditViewFrame *frame,
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
 	gtk_widget_show (menu_item);
 
-	g_object_bind_property (frame->priv->search_settings, "regex-enabled",
-				menu_item, "active",
-				G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+	val = gtk_source_search_settings_get_regex_enabled (frame->priv->search_settings);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), val);
+
+	g_signal_connect (menu_item,
+			  "toggled",
+			  G_CALLBACK (regex_toggled_cb),
+			  frame);
 
 	/* create "Match Entire Word Only" menu item. */
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("Match _Entire Word Only"));
@@ -712,9 +782,13 @@ add_popup_menu_items (GeditViewFrame *frame,
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
 	gtk_widget_show (menu_item);
 
-	g_object_bind_property (frame->priv->search_settings, "at-word-boundaries",
-				menu_item, "active",
-				G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+	val = gtk_source_search_settings_get_at_word_boundaries (frame->priv->search_settings);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), val);
+
+	g_signal_connect (menu_item,
+			  "toggled",
+			  G_CALLBACK (at_word_boundaries_toggled_cb),
+			  frame);
 
 	/* create "Match Case" menu item. */
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("_Match Case"));
@@ -722,9 +796,13 @@ add_popup_menu_items (GeditViewFrame *frame,
 	gtk_menu_shell_prepend (GTK_MENU_SHELL (menu), menu_item);
 	gtk_widget_show (menu_item);
 
-	g_object_bind_property (frame->priv->search_settings, "case-sensitive",
-				menu_item, "active",
-				G_BINDING_BIDIRECTIONAL | G_BINDING_SYNC_CREATE);
+	val = gtk_source_search_settings_get_case_sensitive (frame->priv->search_settings);
+	gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (menu_item), val);
+
+	g_signal_connect (menu_item,
+			  "toggled",
+			  G_CALLBACK (case_sensitive_toggled_cb),
+			  frame);
 }
 
 static void
@@ -1110,7 +1188,6 @@ init_search_entry (GeditViewFrame *frame)
 		gboolean selection_exists;
 		gchar *search_text = NULL;
 		gint selection_len = 0;
-		const gchar *old_search_text;
 		GtkSourceSearchContext *search_context;
 
 		if (frame->priv->search_settings == NULL)
@@ -1121,6 +1198,14 @@ init_search_entry (GeditViewFrame *frame)
 		g_clear_object (&frame->priv->old_search_settings);
 		frame->priv->old_search_settings = copy_search_settings (frame->priv->search_settings);
 
+		g_free (frame->priv->old_search_text);
+		frame->priv->old_search_text = NULL;
+
+		if (frame->priv->search_text != NULL)
+		{
+			frame->priv->old_search_text = g_strdup (frame->priv->search_text);
+		}
+
 		search_context = gtk_source_search_context_new (GTK_SOURCE_BUFFER (buffer),
 								frame->priv->search_settings);
 
@@ -1138,11 +1223,18 @@ init_search_entry (GeditViewFrame *frame)
 		                                      &search_text,
 		                                      &selection_len);
 
-		old_search_text = gtk_source_search_settings_get_search_text (frame->priv->search_settings);
-
 		if (selection_exists && (search_text != NULL) && (selection_len <= 160))
 		{
-			gchar *search_text_escaped = gtk_source_utils_escape_search_text (search_text);
+			gchar *search_text_escaped;
+
+			if (gtk_source_search_settings_get_regex_enabled (frame->priv->search_settings))
+			{
+				search_text_escaped = g_regex_escape_string (search_text, -1);
+			}
+			else
+			{
+				search_text_escaped = gtk_source_utils_escape_search_text (search_text);
+			}
 
 			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
 					    search_text_escaped);
@@ -1152,15 +1244,13 @@ init_search_entry (GeditViewFrame *frame)
 
 			g_free (search_text_escaped);
 		}
-		else if (old_search_text != NULL)
+		else if (frame->priv->search_text != NULL)
 		{
-			gchar *old_search_text_escaped = gtk_source_utils_escape_search_text (old_search_text);
-
 			g_signal_handler_block (frame->priv->search_entry,
 			                        frame->priv->search_entry_changed_id);
 
 			gtk_entry_set_text (GTK_ENTRY (frame->priv->search_entry),
-					    old_search_text_escaped);
+					    frame->priv->search_text);
 
 			gtk_editable_select_region (GTK_EDITABLE (frame->priv->search_entry),
 			                            0, -1);
@@ -1247,6 +1337,7 @@ gedit_view_frame_class_init (GeditViewFrameClass *klass)
 	GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
 
 	object_class->dispose = gedit_view_frame_dispose;
+	object_class->finalize = gedit_view_frame_finalize;
 	object_class->get_property = gedit_view_frame_get_property;
 
 	g_object_class_install_property (object_class, PROP_DOCUMENT,
